package net.sf.javaprinciples.resource;

import java.io.File;
import java.io.FileOutputStream;
import java.io.FileWriter;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.net.URISyntaxException;
import java.security.Principal;
import java.util.Date;

import org.springframework.util.StringUtils;

import net.sf.javaprinciples.core.RequestContext;

/**
 * Save a spring resource to the filesystem
 *
 * @author Luka Kruscic
 */
public class FilesystemPutService implements ResourcePutService<String>
{
    private ResourceMetadataDao resourceMetadataDao;
    private FileResolver fileResolver;

    @Override
    public ResourceIdentifier putResource(ResourceIdentifier identifier, Resource<String> resource) throws ResourceException
    {
        ResourceIdentifier newIdentifier = validateIdentifier(identifier);

        File content = storeContent(newIdentifier, resource);
        storeMetadata(newIdentifier, resource, content);

        return newIdentifier;
    }

    protected ResourceIdentifier validateIdentifier(ResourceIdentifier identifier) throws ResourceException
    {
        String sidentifier = identifier.getIdentifier();
        if (StringUtils.hasText(sidentifier))
        {
            return identifier;
        }
        //TODO Consider injecting an identifier creator
        try
        {
            sidentifier = new URN().toString();
        }
        catch (URISyntaxException e)
        {
            throw new ResourceException("Could not create URN", e);
        }

        return new SimpleResourceIdentifier(sidentifier);
    }

    protected File storeContent(ResourceIdentifier identifier, Resource<String> resource) throws ResourceException
    {
        File file = fileResolver.resolve(identifier);

        if (!file.getParentFile().exists())
        {
            if (!file.getParentFile().mkdirs())
            {
                throw new ResourceException("The resource directory could not be created:" + file.getParentFile().getAbsolutePath());
            }
        }

        String content = resource.getContent();
        FileWriter fileWriter = null;
        try
        {
            OutputStreamWriter writer;
            String fileEncoding = resource.getEncoding();
            if (StringUtils.hasText(fileEncoding))
            {
                writer = new OutputStreamWriter(new FileOutputStream(file), fileEncoding);
            }
            else
            {
                writer = new OutputStreamWriter(new FileOutputStream(file));
            }
            writer.write(content);
            writer.close();
        }
        catch (IOException e)
        {
            throw new ResourceException("Could not store resource", e);
        }
        return file;
    }

    protected void storeMetadata(ResourceIdentifier identifier, Resource<String> resource, File content) throws ResourceException
    {
        ResourceMetadata resourceMetadata = new ResourceMetadata();

        resourceMetadata.setContentLength(resource.getResourceLength());
        resourceMetadata.setCreatedDate(new Date(resource.getValidFrom()));
        resourceMetadata.setExpiryDate(new Date(resource.getValidUntil()));
        resourceMetadata.setContentType(resource.getMimeType());
        resourceMetadata.setFileEncoding(resource.getEncoding());
        resourceMetadata.setIdentifier(identifier.getIdentifier());

        Principal principal = RequestContext.getPrincipal();
        if (principal != null)
        {
            resourceMetadata.setCreator(principal.getName());
        }

        File metadata = new File(content.getParent(), "metadata");
        resourceMetadataDao.store(resourceMetadata, metadata);
    }

    public void setFileResolver(FileResolver fileResolver)
    {
        this.fileResolver = fileResolver;
    }

    public void setResourceMetadataDao(ResourceMetadataDao resourceMetadataDao)
    {
        this.resourceMetadataDao = resourceMetadataDao;
    }
}
