package net.sf.javaprinciples.resource.servlet;

import java.io.InputStream;
import java.io.OutputStream;
import java.util.Map;
import java.util.Set;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.springframework.util.StringUtils;
import org.springframework.web.servlet.View;

import net.sf.javaprinciples.resource.Resource;

public class StreamView implements View
{
    private static final String JSONP_PARAM_NAME = "callback";
    private String contentType;

    /**
     * @return Only null when not injected
     */
    @Override
    public String getContentType()
    {
        return contentType;
    }

    /**
     * @param model Never null
     * @param request Never null
     * @param response Never null
     * @throws Exception never
     */
    @Override
    @SuppressWarnings("unchecked")
    public void render(Map model, HttpServletRequest request, HttpServletResponse response) throws Exception
    {
        ResourceModel<String> resourceModel = (ResourceModel<String>)model.get(ResourceModel.class.getName());

        Set<String> propertyKeys = resourceModel.getPropertyKeys();
        for (String key : propertyKeys)
        {
            response.addHeader(key, resourceModel.getProperty(key));
        }
        
        int statusCode = resourceModel.getStatusCode();
        if (statusCode != 0 && (statusCode < 200 || statusCode >= 300))
        {
            response.sendError(statusCode);
            return;
        }
        if (statusCode != 0)
        {
            response.setStatus(statusCode);
        }
        
        Resource<String> resource = resourceModel.getResource();

        if (resource == null)
        {
            return;
        }

        long validTo = resource.getValidUntil();
        if (validTo > 0)
        {
            long age = validTo - System.currentTimeMillis();
            if (age < 0)
            {
                response.sendError(404);
                return;
            }
            age /= 1000; // Convert from ms to s
            response.addHeader("Cache-Control", "max-age=" + Long.toString(age));
        }

        copyResponseContent(response, request, resource);
    }

    /**
     * Copy the content of the resource into the response and set the appropriate mime-type.
     * 
     * @param response Forms the response to return to the client.
     * @param request Forms the client request.
     * @param resource The requested HTTP resource.
     */
    private void copyResponseContent(
            HttpServletResponse response,
            HttpServletRequest request,
            Resource<String> resource) throws Exception
    {
        // determine the contentType
        String jsonpCallback = request.getParameter(JSONP_PARAM_NAME);
        String responseContentType = null;

        if (jsonpCallback != null)
        {
            responseContentType = "application/json";
        }
        else if (contentType == null)
        {
            responseContentType = resource.getMimeType();
        }
        else
        {
            responseContentType = contentType;
        }
        response.setContentType(responseContentType);

        // Set the encoding after the contentType
        String encoding = resource.getEncoding();
        if (StringUtils.hasText(encoding))
        {
            response.setCharacterEncoding(encoding);
        }

        OutputStream out = response.getOutputStream();
        InputStream in = resource.getInputStream();
        if (in == null)
        {
            return;
        }

        // @TODO select a library method for copying streams.
        // This implementation dropped in until we have time to find something better.
        try
        {
            if (jsonpCallback != null)
            {
                out.write(jsonpCallback.getBytes());
                out.write('(');
            }

            int elem = in.read();
            while (elem != -1)
            {
                out.write(elem);
                elem = in.read();
            }

            if (jsonpCallback != null)
            {
                out.write(')');
            }

            // We dont flush the outputstream as this results in the Content-Length not being set.
        }
        finally
        {
            in.close();
        }
    }
}
